@@ -37,7 +37,7 @@ void printlistfunc(FILE *fout)
   for(i=0;lfunc[i].gpname;i++)
     fprintf(fout,"%s %s %s\n",lfunc[i].gpname,lfunc[i].proto.cname,(lfunc[i].proto.code?lfunc[i].proto.code:""));
 }
-void checkisvar(int aff, char *func, int *var, Gtype *t)
+void checkisvar(int aff, const char *func, int *var, Gtype *t)
 {
   *var=aff;
   switch(tree[*var].f)
@@ -56,7 +56,7 @@ void checkisvar(int aff, char *func, int *var, Gtype *t)
   }
 }
 
-void genequal(int aff, char *func, int *var, int *binf, Gtype *t)
+void genequal(int aff, const char *func, int *var, int *binf, Gtype *t)
 {
   if (tree[aff].f!=Faffect)
     die(aff,"incorrect syntax for %s",func);
@@ -152,11 +152,11 @@ void genblockfuncspec(int n, int p, gpfunc *gp)
   case GPif:
     nb=genlistargs(n,arg,2,3);
     if (nb==3 && arg[2]==GNOARG)
-      /*Some people type 'if(x=0,print(x),)'*/
+      /*Some people type 'if(x!=0,print(x),)'*/
       nb--;
     if (tree[arg[0]].f==Fsmall && tree[arg[0]].x)
     {
-      /*if(1,,) expr are often used to create block. We honor it here*/
+      /*if(1,...) expr are often used to create block. We honor it here*/
       if (p>=0 && tree[p].f!=Fseq)
 	newdecl((1<<Cauto)|(1<<Cconst),Gvoid,-1,&ret);
       arg[1]=geninsertvar(arg[1],ret);
@@ -186,30 +186,37 @@ void genblockfuncspec(int n, int p, gpfunc *gp)
   case GPinstall:
     {
       gpfunc *gp;
-      char *p;
+      const char *p;
       int d, nf;
+      /* We should not use genblock(y,n) since prototype is
+       * {rrD\"\",r,D\"\",s,}
+       * Instead we fix Fentry to Fconst.
+       */
       genblock(y,n);
       nb=genlistargs(n,arg,2,4);
       /*What to do with arg[3]? add a linking ??*/
       if (nb<3 || arg[2]==GNOARG) arg[2]=arg[0];
       d=detag(arg[2]);
-      if (tree[d].f!=Fentry && tree[d].f!=Fstring)
+      if (tree[d].f!=Fentry && !is_const(d,CSTstr))
         die(n,"Incorrect GP name in install");
+      tree[d].f=Fconst;
       nf=getfunc(entryname(d)); gp=lfunc+nf;
       d=detag(arg[0]);
-      if (tree[d].f!=Fentry && tree[d].f!=Fstring)
+      if (tree[d].f!=Fentry && !is_const(d,CSTstr))
         die(n,"Incorrect C name in install");
+      tree[d].f=Fconst;
       gp->proto.cname=entryname(d);
       if (nb==4 && arg[3]!=GNOARG)
       {
 	d=detag(arg[3]);
-	if (tree[d].f!=Fentry && tree[d].f!=Fstring)
+	if (!is_const(d,CSTstr))
 	  die(n,"Incorrect library name in install");
 	gp->proto.origin=entryname(d);
       }
       d=detag(arg[1]);
-      if (tree[d].f!=Fentry && tree[d].f!=Fstring)
+      if (tree[d].f!=Fentry && !is_const(d,CSTstr))
         die(n,"Incorrect code in install");
+      tree[d].f=Fconst;
       p=entryname(d);
       gp->proto.code=p;
       if (gp->proto.code[0]=='v')
@@ -450,7 +457,7 @@ void genblockfuncspec(int n, int p, gpfunc *gp)
     genequaltosmall(binf,bsup,&tv);
     pushvar(var,1<<Cuser,tv,-1);
     seq=geninsertvarop(seq,ret,(gp->spec==GPsum?OPpe:OPme));
-    genblock(arg[2],-1);
+    genblock(seq,-1);
     if (autogc)
     {
       seq=addseqright(seq,newcall("_gerepilemany",gpm));
@@ -761,7 +768,7 @@ int gentypefuncspec(int n, gpfunc *gp)
 }
 void genpercent(FILE *fout, int n)
 {
-  char *s=value[tree[n].x].val.str;
+  const char *s=value[tree[n].x].val.str;
   while(*s)
   {
     if (*s=='%')
@@ -769,50 +776,65 @@ void genpercent(FILE *fout, int n)
     fputc(*s++,fout);
   }
 }
-void genformat(FILE *fout, int nb, int *arg)
+void genformat(FILE *fout, int cnb, int *carg)
 {
-  int i;
-  for(i=0;i<nb;i++)
+  int arg[STACKSZ];
+  int i,j;
+  for(j=0;j<cnb;j++)
   {
-    switch(ctype[tree[arg[i]].t])
+    int nb=listcalltostack(carg[j],OPcat,arg,STACKSZ,"concatenation",carg[j]);
+    for(i=0;i<nb;i++)
     {
+      switch(ctype[tree[arg[i]].t])
+      {
       case Vbptr:
       case Vsmall:
-	if (tree[arg[i]].f!=Fsmall)
-	  fprintf(fout,"%%ld");
-	else
-	  fprintf(fout,"%d",tree[arg[i]].x);
-	break;
+        if (tree[arg[i]].f!=Fsmall)
+          fprintf(fout,"%%ld");
+        else if (tree[arg[i]].t==Gvar)
+          fprintf(fout,"%%Z");
+        else
+          fprintf(fout,"%d",tree[arg[i]].x);
+        break;
       case Vgen:
-	fprintf(fout,"%%Z");
-	break;
+        fprintf(fout,"%%Z");
+        break;
       case Vstr:
-	if (tree[arg[i]].f!=Fstring)
-	  fprintf(fout,"%%s");
-	else
-	  genpercent(fout,arg[i]);
-	break;
+        if (!is_const(arg[i],CSTstr))
+          fprintf(fout,"%%s");
+        else
+          genpercent(fout,arg[i]);
+        break;
       case Vvoid:
-	break;
+        break;
       case Vulong:
-	fprintf(fout,"%%lu");
-	break;
+        fprintf(fout,"%%lu");
+        break;
       case Vgptr:
-	die(-1,"Printing gptr doesn't make sense");
+        die(-1,"Printing gptr doesn't make sense");
+      }
     }
   }
 }
-void genformatarg(FILE *fout, int nb, int *arg)
+void genformatarg(FILE *fout, int cnb, int *carg)
 {
-  int i;
-  for(i=0;i<nb;i++)
-    if (tree[arg[i]].f!=Fstring && tree[arg[i]].f!=Fsmall)
-    {
-      fprintf(fout,",");
-      if (ctype[tree[arg[i]].t]==Vbptr)
-	fprintf(fout,"*");
-      gencode(fout,arg[i]);
-    }
+  int arg[STACKSZ];
+  int i,j;
+  for(j=0;j<cnb;j++)
+  {
+    int nb=listcalltostack(carg[j],OPcat,arg,STACKSZ,"concatenation",carg[j]);
+    for(i=0;i<nb;i++)
+      if (!is_const(arg[i],CSTstr) && tree[arg[i]].f!=Fsmall)
+      {
+        fprintf(fout,",");
+        if (ctype[tree[arg[i]].t]==Vbptr)
+          fprintf(fout,"*");
+        if (tree[arg[i]].t==Gvar)
+          gencast(fout,arg[i],Gpol);
+        else
+          gencode(fout,arg[i]);
+      }
+  }
 } 
 
 void geninitfunc(FILE *fout)
@@ -835,9 +857,9 @@ void geninitfunc(FILE *fout)
 #endif
             genindent(fout);
             if (ctype[ctxstack[i].t]==Vgen)
-              fprintf(fout,"%s = polx[fetch_user_var(\"%s\")];\n",v->var,v->var);
+              fprintf(fout,"%s = polx[fetch_user_var(\"%s\")];\n",v->cvar,v->var);
             else if (ctxstack[i].t==Gvar)
-              fprintf(fout,"%s = fetch_user_var(\"%s\");\n",v->var,v->var);
+              fprintf(fout,"%s = fetch_user_var(\"%s\");\n",v->cvar,v->var);
 #if TEST_UNITIALISED
           }
           break;
@@ -879,7 +901,7 @@ void gencheckargs(FILE *fout)
     {
       doelse=1;
       genindent(fout);
-      fprintf(fout,"if ( !%s ) %s = ",v->var,v->var);
+      fprintf(fout,"if ( !%s ) %s = ",v->cvar,v->cvar);
       gencast(fout,v->initval,v->t);
       fprintf(fout,";\n");
     }
@@ -1245,7 +1267,7 @@ void genentryspec(FILE *fout, int n, gpfunc *gp)
     break;
   case GPprint:
     nb=genlistargs(n,arg,0,STACKSZ);
-    fprintf(fout,"fprintferr(\"");
+    fprintf(fout,"pariputsf(\"");
     genformat(fout,nb,arg);
     if (!isfunc(n,"print1") && !isfunc(n,"printp1"))
       fprintf(fout,"\\n");
@@ -1303,7 +1325,7 @@ void genentryspec(FILE *fout, int n, gpfunc *gp)
     for(i=0;i<bl->g.n;i++)
     {
       if (i) fprintf(fout,", ");
-      fprintf(fout,"&%s",ctxstack[bl->gcvar[i]].var);
+      fprintf(fout,"&%s",ctxstack[bl->gcvar[i]].cvar);
     }
     fprintf(fout," }");
     break;
@@ -1321,7 +1343,7 @@ void genentryspec(FILE *fout, int n, gpfunc *gp)
     }
     else if (bl->gc&(1<<GCupto))
     {
-      char *varname=ctxstack[bl->gcvar[0]].var;
+      const char *varname=ctxstack[bl->gcvar[0]].cvar;
       fprintf(fout,"%s = gerepileupto(",varname);
       gencode(fout,arg[0]);
       fprintf(fout,", %s);\n",varname);
